@@ -1,4 +1,4 @@
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010-2011 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,7 +14,6 @@
 # limitations under the License.
 
 import os
-import re
 
 from webob.exc import HTTPBadRequest, HTTPLengthRequired, \
     HTTPRequestEntityTooLarge
@@ -32,21 +31,30 @@ MAX_META_COUNT = 90
 MAX_META_OVERALL_SIZE = 4096
 #: Max object name length
 MAX_OBJECT_NAME_LENGTH = 1024
+#: Max object list length of a get request for a container
+CONTAINER_LISTING_LIMIT = 10000
+#: Max container list length of a get request for an account
+ACCOUNT_LISTING_LIMIT = 10000
+MAX_ACCOUNT_NAME_LENGTH = 256
+MAX_CONTAINER_NAME_LENGTH = 256
 
 
-def check_metadata(req):
+def check_metadata(req, target_type):
     """
-    Check metadata sent for objects in the request headers.
+    Check metadata sent in the request headers.
 
     :param req: request object
+    :param target_type: str: one of: object, container, or account: indicates
+                        which type the target storage for the metadata is
     :raises HTTPBadRequest: bad metadata
     """
+    prefix = 'x-%s-meta-' % target_type.lower()
     meta_count = 0
     meta_size = 0
     for key, value in req.headers.iteritems():
-        if not key.lower().startswith('x-object-meta-'):
+        if not key.lower().startswith(prefix):
             continue
-        key = key[len('x-object-meta-'):]
+        key = key[len(prefix):]
         if not key:
             return HTTPBadRequest(body='Metadata name cannot be empty',
                     request=req, content_type='text/plain')
@@ -92,6 +100,9 @@ def check_object_creation(req, object_name):
     if req.content_length is None and \
             req.headers.get('transfer-encoding') != 'chunked':
         return HTTPLengthRequired(request=req)
+    if 'X-Copy-From' in req.headers and req.content_length:
+        return HTTPBadRequest(body='Copy requests require a zero byte body',
+                              request=req, content_type='text/plain')
     if len(object_name) > MAX_OBJECT_NAME_LENGTH:
         return HTTPBadRequest(body='Object name length of %d longer than %d' %
                 (len(object_name), MAX_OBJECT_NAME_LENGTH), request=req,
@@ -99,10 +110,21 @@ def check_object_creation(req, object_name):
     if 'Content-Type' not in req.headers:
         return HTTPBadRequest(request=req, content_type='text/plain',
                     body='No content type')
-    if not check_xml_encodable(req.headers['Content-Type']):
+    if not check_utf8(req.headers['Content-Type']):
         return HTTPBadRequest(request=req, body='Invalid Content-Type',
                     content_type='text/plain')
-    return check_metadata(req)
+    if 'x-object-manifest' in req.headers:
+        value = req.headers['x-object-manifest']
+        container = prefix = None
+        try:
+            container, prefix = value.split('/', 1)
+        except ValueError:
+            pass
+        if not container or not prefix or '?' in value or '&' in value or \
+                prefix[0] == '/':
+            return HTTPBadRequest(request=req,
+                body='X-Object-Manifest must in the format container/prefix')
+    return check_metadata(req, 'object')
 
 
 def check_mount(root, drive):
@@ -135,18 +157,18 @@ def check_float(string):
         return False
 
 
-_invalid_xml = re.compile(ur'[^\x09\x0a\x0d\x20-\uD7FF\uE000-\uFFFD%s-%s]' %
-                        (unichr(0x10000), unichr(0x10FFFF)))
-
-
-def check_xml_encodable(string):
+def check_utf8(string):
     """
-    Validate if a string can be encoded in xml.
+    Validate if a string is valid UTF-8 and has no NULL characters.
 
     :param string: string to be validated
-    :returns: True if the string can be encoded in xml, False otherwise
+    :returns: True if the string is valid utf-8 and has no NULL characters,
+              False otherwise
     """
     try:
-        return not _invalid_xml.search(string.decode('UTF-8'))
+        string.decode('UTF-8')
     except UnicodeDecodeError:
         return False
+    if '\x00' in string:
+        return False
+    return True
