@@ -16,9 +16,6 @@
 """
 The swift3 middleware will emulate the S3 REST api on top of swift.
 
-The boto python library is necessary to use this middleware (install
-the python-boto package if you use Ubuntu).
-
 The following opperations are currently supported:
 
     * GET Service
@@ -131,13 +128,23 @@ def canonical_string(req):
     """
     Canonicalize a request to a token that can be signed.
     """
+    amz_headers = {}
+
     buf = "%s\n%s\n%s\n" % (req.method, req.headers.get('Content-MD5', ''),
             req.headers.get('Content-Type') or '')
-    if 'Date' in req.headers:
-        buf += "%s\n" % req.headers['Date']
+
     for amz_header in sorted((key.lower() for key in req.headers
                               if key.lower().startswith('x-amz-'))):
-        buf += "%s:%s\n" % (amz_header, req.headers[amz_header])
+        amz_headers[amz_header] = req.headers[amz_header]
+
+    if 'x-amz-date' in amz_headers:
+        buf += "\n"
+    elif 'Date' in req.headers:
+        buf += "%s\n" % req.headers['Date']
+
+    for k in sorted(key.lower() for key in amz_headers):
+        buf += "%s:%s\n" % (k, amz_headers[k])
+
     path = req.path_qs
     if '?' in path:
         path, args = path.split('?', 1)
@@ -239,7 +246,7 @@ class BucketController(Controller):
 
         if 'acl' in args:
             return get_acl(self.account_name)
-        
+
         objects = loads(''.join(list(body_iter)))
         body = ('<?xml version="1.0" encoding="UTF-8"?>'
             '<ListBucketResult '
@@ -429,7 +436,7 @@ class Swift3Middleware(object):
         self.app = app
 
     def get_controller(self, path):
-        container, obj = split_path(path, 0, 2)
+        container, obj = split_path(path, 0, 2, True)
         d = dict(container_name=container, object_name=obj)
 
         if container and obj:
@@ -438,32 +445,35 @@ class Swift3Middleware(object):
             return BucketController, d
         return ServiceController, d
 
-    def get_account_info(self, env, req):
-        try:
-            account, user, _junk = \
-                req.headers['Authorization'].split(' ')[-1].split(':')
-        except Exception:
-            return None, None
-
-        h = canonical_string(req)
-        token = base64.urlsafe_b64encode(h)
-        return '%s:%s' % (account, user), token
-
     def __call__(self, env, start_response):
         req = Request(env)
-        if not'Authorization' in req.headers:
+
+        if 'AWSAccessKeyId' in req.GET:
+            try:
+                req.headers['Date'] = req.GET['Expires']
+                req.headers['Authorization'] = \
+                    'AWS %(AWSAccessKeyId)s:%(Signature)s' % req.GET
+            except KeyError:
+                return get_err_response('InvalidArgument')(env, start_response)
+
+        if not 'Authorization' in req.headers:
             return self.app(env, start_response)
+
+        try:
+            account, signature = \
+                req.headers['Authorization'].split(' ')[-1].rsplit(':', 1)
+        except Exception:
+            return get_err_response('InvalidArgument')(env, start_response)
+
         try:
             controller, path_parts = self.get_controller(req.path)
         except ValueError:
             return get_err_response('InvalidURI')(env, start_response)
 
-        account_name, token = self.get_account_info(env, req)
-        if not account_name:
-            return get_err_response('InvalidArgument')(env, start_response)
+        token = base64.urlsafe_b64encode(canonical_string(req))
+
+        controller = controller(env, self.app, account, token, **path_parts)
 
-        controller = controller(env, self.app, account_name, token,
-                                **path_parts)
         if hasattr(controller, req.method):
             res = getattr(controller, req.method)(env, start_response)
         else:
