@@ -42,6 +42,7 @@ from eventlet import greenio, GreenPool, sleep, Timeout, listen
 from eventlet.green import socket, subprocess, ssl, thread, threading
 import netifaces
 
+from swift.common.constraints import check_utf8
 from swift.common.exceptions import LockTimeout, MessageTimeout
 
 # logging doesn't import patched as cleanly as one would like
@@ -74,6 +75,8 @@ if hash_conf.read('/etc/swift/swift.conf'):
 # Used when reading config values
 TRUE_VALUES = set(('true', '1', 'yes', 'on', 't', 'y'))
 
+# Used with xml.sax.saxutils.escape
+XML_EXTRA_ENTITIES = dict((chr(x), '&#x%x;' % x) for x in xrange(1, 0x20))
 
 def validate_configuration():
     if HASH_PATH_SUFFIX == '':
@@ -110,8 +113,8 @@ def get_param(req, name, default=None):
     :returns: HTTP request parameter value
     """
     value = req.str_params.get(name, default)
-    if value:
-        value.decode('utf8')    # Ensure UTF8ness
+    if value and not check_utf8(value):
+        raise ValueError('Not valid UTF-8 or contains NULL characters')
     return value
 
 
@@ -144,12 +147,12 @@ def drop_buffer_cache(fd, offset, length):
     """
     global _posix_fadvise
     if _posix_fadvise is None:
-        _posix_fadvise = load_libc_function('posix_fadvise')
+        _posix_fadvise = load_libc_function('posix_fadvise64')
     # 4 means "POSIX_FADV_DONTNEED"
     ret = _posix_fadvise(fd, ctypes.c_uint64(offset),
                         ctypes.c_uint64(length), 4)
     if ret != 0:
-        logging.warn("posix_fadvise(%s, %s, %s, 4) -> %s"
+        logging.warn("posix_fadvise64(%s, %s, %s, 4) -> %s"
                      % (fd, offset, length, ret))
 
 
@@ -972,6 +975,32 @@ def urlparse(url):
     return ModifiedParseResult(*stdlib_urlparse(url))
 
 
+def validate_sync_to(value, allowed_sync_hosts):
+    p = urlparse(value)
+    if p.scheme not in ('http', 'https'):
+        return _('Invalid scheme %r in X-Container-Sync-To, must be "http" '
+                 'or "https".') % p.scheme
+    if not p.path:
+        return _('Path required in X-Container-Sync-To')
+    if p.params or p.query or p.fragment:
+        return _('Params, queries, and fragments not allowed in '
+                 'X-Container-Sync-To')
+    if p.hostname not in allowed_sync_hosts:
+        return _('Invalid host %r in X-Container-Sync-To') % p.hostname
+    return None
+
+
+def get_remote_client(req):
+    # remote host for zeus
+    client = req.headers.get('x-cluster-client-ip')
+    if not client and 'x-forwarded-for' in req.headers:
+        # remote host for other lbs
+        client = req.headers['x-forwarded-for'].split(',')[0].strip()
+    if not client:
+        client = req.remote_addr
+    return client
+
+
 def human_readable(value):
     """
     Returns the number in a human readable format; for example 1048576 = "1Mi".
