@@ -13,29 +13,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from os import environ, kill
+from os import kill
 from signal import SIGTERM
 from subprocess import call, Popen
 from time import sleep
-from ConfigParser import ConfigParser
 
 from swift.common.bufferedhttp import http_connect_raw as http_connect
 from swift.common.client import get_auth
 from swift.common.ring import Ring
 
 
-SUPER_ADMIN_KEY = None
-
-c = ConfigParser()
-PROXY_SERVER_CONF_FILE = environ.get('SWIFT_PROXY_SERVER_CONF_FILE',
-                                     '/etc/swift/proxy-server.conf')
-if c.read(PROXY_SERVER_CONF_FILE):
-    conf = dict(c.items('filter:swauth'))
-    SUPER_ADMIN_KEY = conf.get('super_admin_key', 'swauthkey')
-else:
-    exit('Unable to read config file: %s' % PROXY_SERVER_CONF_FILE)
-
-
 def kill_pids(pids):
     for pid in pids.values():
         try:
@@ -48,8 +35,6 @@ def reset_environment():
     call(['resetswift'])
     pids = {}
     try:
-        pids['proxy'] = Popen(['swift-proxy-server',
-                               '/etc/swift/proxy-server.conf']).pid
         port2server = {}
         for s, p in (('account', 6002), ('container', 6001), ('object', 6000)):
             for n in xrange(1, 5):
@@ -57,14 +42,27 @@ def reset_environment():
                     Popen(['swift-%s-server' % s,
                            '/etc/swift/%s-server/%d.conf' % (s, n)]).pid
                 port2server[p + (n * 10)] = '%s%d' % (s, n)
+        pids['proxy'] = Popen(['swift-proxy-server',
+                               '/etc/swift/proxy-server.conf']).pid
         account_ring = Ring('/etc/swift/account.ring.gz')
         container_ring = Ring('/etc/swift/container.ring.gz')
         object_ring = Ring('/etc/swift/object.ring.gz')
-        sleep(5)
-        call(['recreateaccounts'])
-        url, token = get_auth('http://127.0.0.1:8080/auth/v1.0',
-                              'test:tester', 'testing')
-        account = url.split('/')[-1]
+        attempt = 0
+        while True:
+            attempt += 1
+            try:
+                url, token = get_auth('http://127.0.0.1:8080/auth/v1.0',
+                                      'test:tester', 'testing')
+                account = url.split('/')[-1]
+                break
+            except Exception, err:
+                if attempt > 9:
+                    print err
+                    print 'Giving up after %s retries.' % attempt
+                    raise err
+                print err
+                print 'Retrying in 1 second...'
+                sleep(1)
     except BaseException, err:
         kill_pids(pids)
         raise err
