@@ -1,4 +1,4 @@
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010-2011 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -165,14 +165,14 @@ class TestDatabaseBroker(unittest.TestCase):
         try:
             with broker.get() as conn:
                 conn.execute('SELECT 1')
-        except:
+        except Exception:
             got_exc = True
         broker = DatabaseBroker(os.path.join(self.testdir, '1.db'))
         got_exc = False
         try:
             with broker.get() as conn:
                 conn.execute('SELECT 1')
-        except:
+        except Exception:
             got_exc = True
         self.assert_(got_exc)
         def stub(*args, **kwargs):
@@ -186,7 +186,7 @@ class TestDatabaseBroker(unittest.TestCase):
                 conn.execute('INSERT INTO test (one) VALUES ("1")')
                 raise Exception('test')
                 conn.commit()
-        except:
+        except Exception:
             pass
         broker = DatabaseBroker(os.path.join(self.testdir, '1.db'))
         with broker.get() as conn:
@@ -230,7 +230,7 @@ class TestDatabaseBroker(unittest.TestCase):
         try:
             with broker.lock():
                 raise Exception('test')
-        except:
+        except Exception:
             pass
         with broker.lock():
             pass
@@ -367,6 +367,161 @@ class TestDatabaseBroker(unittest.TestCase):
         broker.merge_syncs([{'sync_point': 5, 'remote_id': uuid2}])
         self.assertEquals(broker.get_sync(uuid2), 5)
 
+    def test_get_replication_info(self):
+        self.get_replication_info_tester(metadata=False)
+
+    def test_get_replication_info_with_metadata(self):
+        self.get_replication_info_tester(metadata=True)
+
+    def get_replication_info_tester(self, metadata=False):
+        broker = DatabaseBroker(':memory:', account='a')
+        broker.db_type = 'test'
+        broker.db_contains_type = 'test'
+        broker_creation = normalize_timestamp(1)
+        broker_uuid = str(uuid4())
+        broker_metadata = metadata and simplejson.dumps(
+                {'Test': ('Value', normalize_timestamp(1))}) or ''
+        def _initialize(conn, put_timestamp):
+            if put_timestamp is None:
+                put_timestamp = normalize_timestamp(0)
+            conn.executescript('''
+                CREATE TABLE test (
+                    ROWID INTEGER PRIMARY KEY AUTOINCREMENT,
+                    name TEXT UNIQUE,
+                    created_at TEXT
+                );
+                CREATE TRIGGER test_insert AFTER INSERT ON test
+                BEGIN
+                    UPDATE test_stat
+                    SET test_count = test_count + 1,
+                        hash = chexor(hash, new.name, new.created_at);
+                END;
+                CREATE TRIGGER test_update BEFORE UPDATE ON test
+                BEGIN
+                    SELECT RAISE(FAIL,
+                                 'UPDATE not allowed; DELETE and INSERT');
+                END;
+                CREATE TRIGGER test_delete AFTER DELETE ON test
+                BEGIN
+                    UPDATE test_stat
+                    SET test_count = test_count - 1,
+                        hash = chexor(hash, old.name, old.created_at);
+                END;
+                CREATE TABLE test_stat (
+                    account TEXT,
+                    created_at TEXT,
+                    put_timestamp TEXT DEFAULT '0',
+                    delete_timestamp TEXT DEFAULT '0',
+                    test_count INTEGER,
+                    hash TEXT default '00000000000000000000000000000000',
+                    id TEXT
+                    %s
+                );
+                INSERT INTO test_stat (test_count) VALUES (0);
+            ''' % (metadata and ", metadata TEXT DEFAULT ''" or ""))
+            conn.execute('''
+                UPDATE test_stat
+                SET account = ?, created_at = ?,  id = ?, put_timestamp = ?
+            ''', (broker.account, broker_creation, broker_uuid, put_timestamp))
+            if metadata:
+                conn.execute('UPDATE test_stat SET metadata = ?',
+                             (broker_metadata,))
+            conn.commit()
+        broker._initialize = _initialize
+        put_timestamp = normalize_timestamp(2)
+        broker.initialize(put_timestamp)
+        info = broker.get_replication_info()
+        self.assertEquals(info, {'count': 0,
+            'hash': '00000000000000000000000000000000',
+            'created_at': broker_creation, 'put_timestamp': put_timestamp,
+            'delete_timestamp': '0', 'max_row': -1, 'id': broker_uuid,
+            'metadata': broker_metadata})
+        insert_timestamp = normalize_timestamp(3)
+        with broker.get() as conn:
+            conn.execute('''
+                INSERT INTO test (name, created_at) VALUES ('test', ?)
+            ''', (insert_timestamp,))
+            conn.commit()
+        info = broker.get_replication_info()
+        self.assertEquals(info, {'count': 1,
+            'hash': 'bdc4c93f574b0d8c2911a27ce9dd38ba',
+            'created_at': broker_creation, 'put_timestamp': put_timestamp,
+            'delete_timestamp': '0', 'max_row': 1, 'id': broker_uuid,
+            'metadata': broker_metadata})
+        with broker.get() as conn:
+            conn.execute('DELETE FROM test')
+            conn.commit()
+        info = broker.get_replication_info()
+        self.assertEquals(info, {'count': 0,
+            'hash': '00000000000000000000000000000000',
+            'created_at': broker_creation, 'put_timestamp': put_timestamp,
+            'delete_timestamp': '0', 'max_row': 1, 'id': broker_uuid,
+            'metadata': broker_metadata})
+        return broker
+
+    def test_metadata(self):
+        # Initializes a good broker for us
+        broker = self.get_replication_info_tester(metadata=True)
+        # Add our first item
+        first_timestamp = normalize_timestamp(1)
+        first_value = '1'
+        broker.update_metadata({'First': [first_value, first_timestamp]})
+        self.assert_('First' in broker.metadata)
+        self.assertEquals(broker.metadata['First'],
+                          [first_value, first_timestamp])
+        # Add our second item
+        second_timestamp = normalize_timestamp(2)
+        second_value = '2'
+        broker.update_metadata({'Second': [second_value, second_timestamp]})
+        self.assert_('First' in broker.metadata)
+        self.assertEquals(broker.metadata['First'],
+                          [first_value, first_timestamp])
+        self.assert_('Second' in broker.metadata)
+        self.assertEquals(broker.metadata['Second'],
+                          [second_value, second_timestamp])
+        # Update our first item
+        first_timestamp = normalize_timestamp(3)
+        first_value = '1b'
+        broker.update_metadata({'First': [first_value, first_timestamp]})
+        self.assert_('First' in broker.metadata)
+        self.assertEquals(broker.metadata['First'],
+                          [first_value, first_timestamp])
+        self.assert_('Second' in broker.metadata)
+        self.assertEquals(broker.metadata['Second'],
+                          [second_value, second_timestamp])
+        # Delete our second item (by setting to empty string)
+        second_timestamp = normalize_timestamp(4)
+        second_value = ''
+        broker.update_metadata({'Second': [second_value, second_timestamp]})
+        self.assert_('First' in broker.metadata)
+        self.assertEquals(broker.metadata['First'],
+                          [first_value, first_timestamp])
+        self.assert_('Second' in broker.metadata)
+        self.assertEquals(broker.metadata['Second'],
+                          [second_value, second_timestamp])
+        # Reclaim at point before second item was deleted
+        broker.reclaim(normalize_timestamp(3))
+        self.assert_('First' in broker.metadata)
+        self.assertEquals(broker.metadata['First'],
+                          [first_value, first_timestamp])
+        self.assert_('Second' in broker.metadata)
+        self.assertEquals(broker.metadata['Second'],
+                          [second_value, second_timestamp])
+        # Reclaim at point second item was deleted
+        broker.reclaim(normalize_timestamp(4))
+        self.assert_('First' in broker.metadata)
+        self.assertEquals(broker.metadata['First'],
+                          [first_value, first_timestamp])
+        self.assert_('Second' in broker.metadata)
+        self.assertEquals(broker.metadata['Second'],
+                          [second_value, second_timestamp])
+        # Reclaim after point second item was deleted
+        broker.reclaim(normalize_timestamp(5))
+        self.assert_('First' in broker.metadata)
+        self.assertEquals(broker.metadata['First'],
+                          [first_value, first_timestamp])
+        self.assert_('Second' not in broker.metadata)
+
 
 class TestContainerBroker(unittest.TestCase):
     """ Tests for swift.common.db.ContainerBroker """
@@ -393,7 +548,7 @@ class TestContainerBroker(unittest.TestCase):
             with broker.get() as conn:
                 self.assertEquals(first_conn, conn)
                 raise Exception('OMG')
-        except:
+        except Exception:
             pass
         self.assert_(broker.conn == None)
 
@@ -823,52 +978,57 @@ class TestContainerBroker(unittest.TestCase):
                               normalize_timestamp(time()), 0, 'text/plain',
                               'd41d8cd98f00b204e9800998ecf8427e')
 
-        listing = broker.list_objects_iter(100, '', None, '')
+        listing = broker.list_objects_iter(100, '', None, None, '')
         self.assertEquals(len(listing), 100)
         self.assertEquals(listing[0][0], '0/0000')
         self.assertEquals(listing[-1][0], '0/0099')
 
-        listing = broker.list_objects_iter(100, '0/0099', None, '')
+        listing = broker.list_objects_iter(100, '', '0/0050', None, '')
+        self.assertEquals(len(listing), 50)
+        self.assertEquals(listing[0][0], '0/0000')
+        self.assertEquals(listing[-1][0], '0/0049')
+
+        listing = broker.list_objects_iter(100, '0/0099', None, None, '')
         self.assertEquals(len(listing), 100)
         self.assertEquals(listing[0][0], '0/0100')
         self.assertEquals(listing[-1][0], '1/0074')
 
-        listing = broker.list_objects_iter(55, '1/0074', None, '')
+        listing = broker.list_objects_iter(55, '1/0074', None, None, '')
         self.assertEquals(len(listing), 55)
         self.assertEquals(listing[0][0], '1/0075')
         self.assertEquals(listing[-1][0], '2/0004')
 
-        listing = broker.list_objects_iter(10, '', '0/01', '')
+        listing = broker.list_objects_iter(10, '', None, '0/01', '')
         self.assertEquals(len(listing), 10)
         self.assertEquals(listing[0][0], '0/0100')
         self.assertEquals(listing[-1][0], '0/0109')
 
-        listing = broker.list_objects_iter(10, '', '0/', '/')
+        listing = broker.list_objects_iter(10, '', None, '0/', '/')
         self.assertEquals(len(listing), 10)
         self.assertEquals(listing[0][0], '0/0000')
         self.assertEquals(listing[-1][0], '0/0009')
 
-        listing = broker.list_objects_iter(10, '', '', '/')
+        listing = broker.list_objects_iter(10, '', None, '', '/')
         self.assertEquals(len(listing), 4)
         self.assertEquals([row[0] for row in listing],
                           ['0/', '1/', '2/', '3/'])
 
-        listing = broker.list_objects_iter(10, '2', None, '/')
+        listing = broker.list_objects_iter(10, '2', None, None, '/')
         self.assertEquals(len(listing), 2)
         self.assertEquals([row[0] for row in listing], ['2/', '3/'])
 
-        listing = broker.list_objects_iter(10, '2/', None, '/')
+        listing = broker.list_objects_iter(10, '2/',None,  None, '/')
         self.assertEquals(len(listing), 1)
         self.assertEquals([row[0] for row in listing], ['3/'])
 
-        listing = broker.list_objects_iter(10, '2/0050', '2/', '/')
+        listing = broker.list_objects_iter(10, '2/0050', None, '2/', '/')
         self.assertEquals(len(listing), 10)
         self.assertEquals(listing[0][0], '2/0051')
         self.assertEquals(listing[1][0], '2/0051/')
         self.assertEquals(listing[2][0], '2/0052')
         self.assertEquals(listing[-1][0], '2/0059')
 
-        listing = broker.list_objects_iter(10, '3/0045', '3/', '/')
+        listing = broker.list_objects_iter(10, '3/0045', None, '3/', '/')
         self.assertEquals(len(listing), 10)
         self.assertEquals([row[0] for row in listing],
                            ['3/0045/', '3/0046', '3/0046/', '3/0047',
@@ -877,33 +1037,34 @@ class TestContainerBroker(unittest.TestCase):
 
         broker.put_object('3/0049/', normalize_timestamp(time()), 0,
                           'text/plain', 'd41d8cd98f00b204e9800998ecf8427e')
-        listing = broker.list_objects_iter(10, '3/0048', None, None)
+        listing = broker.list_objects_iter(10, '3/0048', None, None, None)
         self.assertEquals(len(listing), 10)
         self.assertEquals([row[0] for row in listing],
             ['3/0048/0049', '3/0049', '3/0049/',
             '3/0049/0049', '3/0050', '3/0050/0049', '3/0051', '3/0051/0049',
             '3/0052', '3/0052/0049'])
 
-        listing = broker.list_objects_iter(10, '3/0048', '3/', '/')
+        listing = broker.list_objects_iter(10, '3/0048', None, '3/', '/')
         self.assertEquals(len(listing), 10)
         self.assertEquals([row[0] for row in listing],
             ['3/0048/', '3/0049', '3/0049/', '3/0050',
             '3/0050/', '3/0051', '3/0051/', '3/0052', '3/0052/', '3/0053'])
 
-        listing = broker.list_objects_iter(10, None, '3/0049/', '/')
+        listing = broker.list_objects_iter(10, None, None, '3/0049/', '/')
         self.assertEquals(len(listing), 2)
         self.assertEquals([row[0] for row in listing],
             ['3/0049/', '3/0049/0049'])
 
-        listing = broker.list_objects_iter(10, None, None, None, '3/0049')
+        listing = broker.list_objects_iter(10, None, None, None, None,
+                                           '3/0049')
         self.assertEquals(len(listing), 1)
         self.assertEquals([row[0] for row in listing], ['3/0049/0049'])
 
-        listing = broker.list_objects_iter(2, None, '3/', '/')
+        listing = broker.list_objects_iter(2, None, None, '3/', '/')
         self.assertEquals(len(listing), 2)
         self.assertEquals([row[0] for row in listing], ['3/0000', '3/0000/'])
 
-        listing = broker.list_objects_iter(2, None, None, None, '3')
+        listing = broker.list_objects_iter(2, None, None, None, None, '3')
         self.assertEquals(len(listing), 2)
         self.assertEquals([row[0] for row in listing], ['3/0000', '3/0001'])
 
@@ -927,11 +1088,11 @@ class TestContainerBroker(unittest.TestCase):
 
         #def list_objects_iter(self, limit, marker, prefix, delimiter, path=None,
         #                          format=None):
-        listing = broker.list_objects_iter(100, None, '/pets/f', '/')
+        listing = broker.list_objects_iter(100, None, None, '/pets/f', '/')
         self.assertEquals([row[0] for row in listing], ['/pets/fish/', '/pets/fish_info.txt'])
-        listing = broker.list_objects_iter(100, None, '/pets/fish', '/')
+        listing = broker.list_objects_iter(100, None, None, '/pets/fish', '/')
         self.assertEquals([row[0] for row in listing], ['/pets/fish/', '/pets/fish_info.txt'])
-        listing = broker.list_objects_iter(100, None, '/pets/fish/', '/')
+        listing = broker.list_objects_iter(100, None, None, '/pets/fish/', '/')
         self.assertEquals([row[0] for row in listing], ['/pets/fish/a', '/pets/fish/b'])
 
     def test_double_check_trailing_delimiter(self):
@@ -959,19 +1120,19 @@ class TestContainerBroker(unittest.TestCase):
                           'text/plain', 'd41d8cd98f00b204e9800998ecf8427e')
         broker.put_object('c', normalize_timestamp(time()), 0,
                           'text/plain', 'd41d8cd98f00b204e9800998ecf8427e')
-        listing = broker.list_objects_iter(15, None, None, None)
+        listing = broker.list_objects_iter(15, None, None, None, None)
         self.assertEquals(len(listing), 10)
         self.assertEquals([row[0] for row in listing],
             ['a', 'a/', 'a/a', 'a/a/a', 'a/a/b', 'a/b', 'b', 'b/a', 'b/b', 'c'])
-        listing = broker.list_objects_iter(15, None, '', '/')
+        listing = broker.list_objects_iter(15, None, None, '', '/')
         self.assertEquals(len(listing), 5)
         self.assertEquals([row[0] for row in listing],
             ['a', 'a/', 'b', 'b/', 'c'])
-        listing = broker.list_objects_iter(15, None, 'a/', '/')
+        listing = broker.list_objects_iter(15, None, None, 'a/', '/')
         self.assertEquals(len(listing), 4)
         self.assertEquals([row[0] for row in listing],
             ['a/', 'a/a', 'a/a/', 'a/b'])
-        listing = broker.list_objects_iter(15, None, 'b/', '/')
+        listing = broker.list_objects_iter(15, None, None, 'b/', '/')
         self.assertEquals(len(listing), 2)
         self.assertEquals([row[0] for row in listing], ['b/a', 'b/b'])
 
@@ -1119,6 +1280,78 @@ class TestContainerBroker(unittest.TestCase):
                 self.assertEquals(rec['content_type'], 'text/plain')
 
 
+def premetadata_create_container_stat_table(self, conn, put_timestamp=None):
+    """
+    Copied from swift.common.db.ContainerBroker before the metadata column was
+    added; used for testing with TestContainerBrokerBeforeMetadata.
+
+    Create the container_stat table which is specifc to the container DB.
+
+    :param conn: DB connection object
+    :param put_timestamp: put timestamp
+    """
+    if put_timestamp is None:
+        put_timestamp = normalize_timestamp(0)
+    conn.executescript("""
+        CREATE TABLE container_stat (
+            account TEXT,
+            container TEXT,
+            created_at TEXT,
+            put_timestamp TEXT DEFAULT '0',
+            delete_timestamp TEXT DEFAULT '0',
+            object_count INTEGER,
+            bytes_used INTEGER,
+            reported_put_timestamp TEXT DEFAULT '0',
+            reported_delete_timestamp TEXT DEFAULT '0',
+            reported_object_count INTEGER DEFAULT 0,
+            reported_bytes_used INTEGER DEFAULT 0,
+            hash TEXT default '00000000000000000000000000000000',
+            id TEXT,
+            status TEXT DEFAULT '',
+            status_changed_at TEXT DEFAULT '0'
+        );
+
+        INSERT INTO container_stat (object_count, bytes_used)
+            VALUES (0, 0);
+    """)
+    conn.execute('''
+        UPDATE container_stat
+        SET account = ?, container = ?, created_at = ?, id = ?,
+            put_timestamp = ?
+    ''', (self.account, self.container, normalize_timestamp(time()),
+          str(uuid4()), put_timestamp))
+
+
+class TestContainerBrokerBeforeMetadata(TestContainerBroker):
+    """
+    Tests for swift.common.db.ContainerBroker against databases created before
+    the metadata column was added.
+    """
+
+    def setUp(self):
+        self._imported_create_container_stat_table = \
+            ContainerBroker.create_container_stat_table
+        ContainerBroker.create_container_stat_table = \
+            premetadata_create_container_stat_table
+        broker = ContainerBroker(':memory:', account='a', container='c')
+        broker.initialize(normalize_timestamp('1'))
+        exc = None
+        with broker.get() as conn:
+            try:
+                conn.execute('SELECT metadata FROM container_stat')
+            except BaseException, err:
+                exc = err
+        self.assert_('no such column: metadata' in str(exc))
+
+    def tearDown(self):
+        ContainerBroker.create_container_stat_table = \
+            self._imported_create_container_stat_table
+        broker = ContainerBroker(':memory:', account='a', container='c')
+        broker.initialize(normalize_timestamp('1'))
+        with broker.get() as conn:
+            conn.execute('SELECT metadata FROM container_stat')
+
+
 class TestAccountBroker(unittest.TestCase):
     """ Tests for swift.common.db.AccountBroker """
 
@@ -1130,7 +1363,7 @@ class TestAccountBroker(unittest.TestCase):
         try:
             with broker.get() as conn:
                 pass
-        except:
+        except Exception:
             got_exc = True
         self.assert_(got_exc)
         broker.initialize(normalize_timestamp('1'))
@@ -1151,7 +1384,7 @@ class TestAccountBroker(unittest.TestCase):
             with broker.get() as conn:
                 self.assertEquals(first_conn, conn)
                 raise Exception('OMG')
-        except:
+        except Exception:
             pass
         self.assert_(broker.conn == None)
 
@@ -1419,57 +1652,62 @@ class TestAccountBroker(unittest.TestCase):
             broker.put_container('3/%04d/0049' % cont,
                                  normalize_timestamp(time()), 0, 0, 0)
 
-        listing = broker.list_containers_iter(100, '', None, '')
+        listing = broker.list_containers_iter(100, '', None, None, '')
         self.assertEquals(len(listing), 100)
         self.assertEquals(listing[0][0], '0/0000')
         self.assertEquals(listing[-1][0], '0/0099')
 
-        listing = broker.list_containers_iter(100, '0/0099', None, '')
+        listing = broker.list_containers_iter(100, '', '0/0050', None, '')
+        self.assertEquals(len(listing), 51)
+        self.assertEquals(listing[0][0], '0/0000')
+        self.assertEquals(listing[-1][0], '0/0050')
+
+        listing = broker.list_containers_iter(100, '0/0099', None, None, '')
         self.assertEquals(len(listing), 100)
         self.assertEquals(listing[0][0], '0/0100')
         self.assertEquals(listing[-1][0], '1/0074')
 
-        listing = broker.list_containers_iter(55, '1/0074', None, '')
+        listing = broker.list_containers_iter(55, '1/0074', None, None, '')
         self.assertEquals(len(listing), 55)
         self.assertEquals(listing[0][0], '1/0075')
         self.assertEquals(listing[-1][0], '2/0004')
 
-        listing = broker.list_containers_iter(10, '', '0/01', '')
+        listing = broker.list_containers_iter(10, '', None, '0/01', '')
         self.assertEquals(len(listing), 10)
         self.assertEquals(listing[0][0], '0/0100')
         self.assertEquals(listing[-1][0], '0/0109')
 
-        listing = broker.list_containers_iter(10, '', '0/01', '/')
+        listing = broker.list_containers_iter(10, '', None, '0/01', '/')
         self.assertEquals(len(listing), 10)
         self.assertEquals(listing[0][0], '0/0100')
         self.assertEquals(listing[-1][0], '0/0109')
 
-        listing = broker.list_containers_iter(10, '', '0/', '/')
+        listing = broker.list_containers_iter(10, '', None, '0/', '/')
         self.assertEquals(len(listing), 10)
         self.assertEquals(listing[0][0], '0/0000')
         self.assertEquals(listing[-1][0], '0/0009')
 
-        listing = broker.list_containers_iter(10, '', '', '/')
+        listing = broker.list_containers_iter(10, '', None, '', '/')
         self.assertEquals(len(listing), 4)
         self.assertEquals([row[0] for row in listing],
                           ['0/', '1/', '2/', '3/'])
 
-        listing = broker.list_containers_iter(10, '2/', None, '/')
+        listing = broker.list_containers_iter(10, '2/', None, None, '/')
         self.assertEquals(len(listing), 1)
         self.assertEquals([row[0] for row in listing], ['3/'])
 
-        listing = broker.list_containers_iter(10, '', '2', '/')
+        listing = broker.list_containers_iter(10, '', None, '2', '/')
         self.assertEquals(len(listing), 1)
         self.assertEquals([row[0] for row in listing], ['2/'])
 
-        listing = broker.list_containers_iter(10, '2/0050', '2/', '/')
+        listing = broker.list_containers_iter(10, '2/0050', None, '2/', '/')
         self.assertEquals(len(listing), 10)
         self.assertEquals(listing[0][0], '2/0051')
         self.assertEquals(listing[1][0], '2/0051/')
         self.assertEquals(listing[2][0], '2/0052')
         self.assertEquals(listing[-1][0], '2/0059')
 
-        listing = broker.list_containers_iter(10, '3/0045', '3/', '/')
+        listing = broker.list_containers_iter(10, '3/0045', None, '3/', '/')
         self.assertEquals(len(listing), 10)
         self.assertEquals([row[0] for row in listing],
                            ['3/0045/', '3/0046', '3/0046/', '3/0047',
@@ -1477,21 +1715,21 @@ class TestAccountBroker(unittest.TestCase):
                             '3/0049/', '3/0050'])
 
         broker.put_container('3/0049/', normalize_timestamp(time()), 0, 0, 0)
-        listing = broker.list_containers_iter(10, '3/0048', None, None)
+        listing = broker.list_containers_iter(10, '3/0048', None, None, None)
         self.assertEquals(len(listing), 10)
         self.assertEquals([row[0] for row in listing],
                            ['3/0048/0049', '3/0049', '3/0049/', '3/0049/0049',
                             '3/0050', '3/0050/0049', '3/0051', '3/0051/0049',
                             '3/0052', '3/0052/0049'])
 
-        listing = broker.list_containers_iter(10, '3/0048', '3/', '/')
+        listing = broker.list_containers_iter(10, '3/0048', None, '3/', '/')
         self.assertEquals(len(listing), 10)
         self.assertEquals([row[0] for row in listing],
                            ['3/0048/', '3/0049', '3/0049/', '3/0050',
                             '3/0050/', '3/0051', '3/0051/', '3/0052',
                             '3/0052/', '3/0053'])
 
-        listing = broker.list_containers_iter(10, None, '3/0049/', '/')
+        listing = broker.list_containers_iter(10, None, None, '3/0049/', '/')
         self.assertEquals(len(listing), 2)
         self.assertEquals([row[0] for row in listing],
                           ['3/0049/', '3/0049/0049'])
@@ -1511,20 +1749,20 @@ class TestAccountBroker(unittest.TestCase):
         broker.put_container('b/a', normalize_timestamp(time()), 0, 0, 0)
         broker.put_container('b/b', normalize_timestamp(time()), 0, 0, 0)
         broker.put_container('c', normalize_timestamp(time()), 0, 0, 0)
-        listing = broker.list_containers_iter(15, None, None, None)
+        listing = broker.list_containers_iter(15, None, None, None, None)
         self.assertEquals(len(listing), 10)
         self.assertEquals([row[0] for row in listing],
                            ['a', 'a/', 'a/a', 'a/a/a', 'a/a/b', 'a/b', 'b',
                             'b/a', 'b/b', 'c'])
-        listing = broker.list_containers_iter(15, None, '', '/')
+        listing = broker.list_containers_iter(15, None, None, '', '/')
         self.assertEquals(len(listing), 5)
         self.assertEquals([row[0] for row in listing],
                           ['a', 'a/', 'b', 'b/', 'c'])
-        listing = broker.list_containers_iter(15, None, 'a/', '/')
+        listing = broker.list_containers_iter(15, None, None, 'a/', '/')
         self.assertEquals(len(listing), 4)
         self.assertEquals([row[0] for row in listing],
                           ['a/', 'a/a', 'a/a/', 'a/b'])
-        listing = broker.list_containers_iter(15, None, 'b/', '/')
+        listing = broker.list_containers_iter(15, None, None, 'b/', '/')
         self.assertEquals(len(listing), 2)
         self.assertEquals([row[0] for row in listing], ['b/a', 'b/b'])
 
@@ -1575,5 +1813,70 @@ class TestAccountBroker(unittest.TestCase):
                           sorted([rec['name'] for rec in items]))
 
 
+def premetadata_create_account_stat_table(self, conn, put_timestamp):
+    """
+    Copied from swift.common.db.AccountBroker before the metadata column was
+    added; used for testing with TestAccountBrokerBeforeMetadata.
+
+    Create account_stat table which is specific to the account DB.
+
+    :param conn: DB connection object
+    :param put_timestamp: put timestamp
+    """
+    conn.executescript("""
+        CREATE TABLE account_stat (
+            account TEXT,
+            created_at TEXT,
+            put_timestamp TEXT DEFAULT '0',
+            delete_timestamp TEXT DEFAULT '0',
+            container_count INTEGER,
+            object_count INTEGER DEFAULT 0,
+            bytes_used INTEGER DEFAULT 0,
+            hash TEXT default '00000000000000000000000000000000',
+            id TEXT,
+            status TEXT DEFAULT '',
+            status_changed_at TEXT DEFAULT '0'
+        );
+
+        INSERT INTO account_stat (container_count) VALUES (0);
+    """)
+
+    conn.execute('''
+        UPDATE account_stat SET account = ?, created_at = ?, id = ?,
+               put_timestamp = ?
+        ''', (self.account, normalize_timestamp(time()), str(uuid4()),
+        put_timestamp))
+
+
+class TestAccountBrokerBeforeMetadata(TestAccountBroker):
+    """
+    Tests for swift.common.db.AccountBroker against databases created before
+    the metadata column was added.
+    """
+
+    def setUp(self):
+        self._imported_create_account_stat_table = \
+            AccountBroker.create_account_stat_table
+        AccountBroker.create_account_stat_table = \
+            premetadata_create_account_stat_table
+        broker = AccountBroker(':memory:', account='a')
+        broker.initialize(normalize_timestamp('1'))
+        exc = None
+        with broker.get() as conn:
+            try:
+                conn.execute('SELECT metadata FROM account_stat')
+            except BaseException, err:
+                exc = err
+        self.assert_('no such column: metadata' in str(exc))
+
+    def tearDown(self):
+        AccountBroker.create_account_stat_table = \
+            self._imported_create_account_stat_table
+        broker = AccountBroker(':memory:', account='a')
+        broker.initialize(normalize_timestamp('1'))
+        with broker.get() as conn:
+            conn.execute('SELECT metadata FROM account_stat')
+
+
 if __name__ == '__main__':
     unittest.main()
